@@ -1,10 +1,10 @@
-(* $Id: pxp_dtd.ml 713 2005-05-04 13:42:45Z gerd $
+(* $Id: pxp_dtd.ml 738 2009-02-02 03:27:35Z gerd $
  * ----------------------------------------------------------------------
  * PXP: The polymorphic XML parser for Objective Caml.
  * Copyright by Gerd Stolpmann. See LICENSE for details.
  *)
 
-open Pxp_core_types
+open Pxp_core_types.I
 open Pxp_lexer_types
 open Pxp_lexers
 open Pxp_entity
@@ -511,7 +511,7 @@ class dtd  ?swarner the_warner init_encoding =
 
     method pinstr_names = pinstr_names
 
-    method write_ref os enc =
+    method write_ref ?root:proot os enc =
       let write_sysid s =
 	write_markup_string 
 	  ~from_enc:`Enc_utf8 ~to_enc:enc os
@@ -525,8 +525,12 @@ class dtd  ?swarner the_warner init_encoding =
 	write_markup_string ~from_enc:encoding ~to_enc:enc os in
 
       wms "<!DOCTYPE ";
-      ( match root with
-	    None -> failwith "#write: DTD without root";
+      ( match proot with
+	  | None ->
+	      ( match root with
+		    None -> failwith "#write: DTD without root";
+		  | Some r -> wms r
+	      )
 	  | Some r -> wms r
       );
       begin match id with
@@ -550,7 +554,7 @@ class dtd  ?swarner the_warner init_encoding =
       wms ">\n";
 
 
-    method write os enc doctype = 
+    method write ?root:proot os enc doctype = 
       let wms = 
 	write_markup_string ~from_enc:encoding ~to_enc:enc os in
 
@@ -566,9 +570,13 @@ class dtd  ?swarner the_warner init_encoding =
 
       if doctype then begin
 	wms "<!DOCTYPE ";
-	( match root with
-	    None -> failwith "#write: DTD without root";
-	  | Some r -> wms r
+	( match proot with
+	    | None ->
+		( match root with
+		      None -> failwith "#write: DTD without root";
+		    | Some r -> wms r
+		);
+	    | Some r -> wms r
 	);
 	wms " [\n";
       end;
@@ -666,7 +674,7 @@ class dtd  ?swarner the_warner init_encoding =
 	       try Str_hashtbl.find gen_entities name with Not_found -> assert false 
 	     in
 	     if ent # is_ndata then begin
-	       let xid = ent # ext_id in
+	       let _xid = ent # ext_id in
 	       let notation = ent # notation in
 	       try
 		 ignore(self # notation notation)
@@ -1187,7 +1195,7 @@ and dtd_notation the_name the_xid init_encoding =
 object (self)
     val name = the_name
     val xid = (the_xid : ext_id)
-    val encoding = (init_encoding : Pxp_core_types.rep_encoding)
+    val encoding = (init_encoding : Pxp_core_types.I.rep_encoding)
     method name = name
     method ext_id = xid
     method encoding = encoding
@@ -1228,7 +1236,7 @@ and proc_instruction the_target the_value init_encoding =
 object (self)
     val target = the_target
     val value = (the_value : string)
-    val encoding = (init_encoding : Pxp_core_types.rep_encoding)
+    val encoding = (init_encoding : Pxp_core_types.I.rep_encoding)
 
     initializer
       match target with
@@ -1318,12 +1326,20 @@ module Entity = struct
       | Entity(make,resolver) ->
 	  make dtd  (* resolver ignored *)
 
-  let entity_id ent = (ent :> < >)
+  let entity_id ent = (ent :> entity_id)
 
-  class fake = object end
+  class fake = object method pxp_magic_coercion() : unit = raise Not_found end
 
   let create_entity_id () = new fake
 
+  let lookup eid =
+    try 
+      let () = eid#pxp_magic_coercion() in
+      assert false
+    with
+      | Not_found -> invalid_arg "Pxp_dtd.Event.lookup"
+      | Pxp_entity.Coerced_entity e -> e
+
 end
 
 
