@@ -1,4 +1,4 @@
-(* $Id: pxp_dtd_parser.ml 696 2004-08-20 14:49:57Z gerd $
+(* $Id: pxp_dtd_parser.ml 738 2009-02-02 03:27:35Z gerd $
  * ----------------------------------------------------------------------
  * PXP: The polymorphic XML parser for Objective Caml.
  * Copyright by Gerd Stolpmann. See LICENSE for details.
@@ -24,7 +24,7 @@ object (self)
    * because this is impossible for `Entry_declarations.
    *)
 
-  method private init_for_xml_body() = 
+  method private init_for_xml_body _ = 
     assert false
 
   method private event_document_xmldecl xmldecl = 
